#include "stdafx.h"
#include "BiFile.h"

static const WCHAR bom = L'\xFEFF';

/****************************************************************************
*                              CBiFile::WriteBOM
* Result: void
*
* Effect: 
*       If in Unicode mode and UTF-16 output, write BOM (Byte Order Marker
*       If in ANSI mode, does nothing
* Throws:
*       CFIleException * if an error
****************************************************************************/

void CBiFile::WriteBOM()
    {
#ifndef _UNICODE
     return; // non-Unicode always succeeds, no exception thrown
#else
     if(mode == 16)
        { /* BOM */
         CFile::Write(&bom, sizeof(bom));
        } /* BOM */
#endif
    } // CBiFile::WriteBOM

/****************************************************************************
*                                CBiFile::Open
* Inputs:
*       LPCTSTR filename:
*       UINT flags: CFile::mode-whatever
*       UINT m: Desired translation mode
* Result: BOOL
*       TRUE if successful
*       FALSE if error (use ::GetLastError)
* Effect: 
*       Opens the file in the desired mode
****************************************************************************/

BOOL CBiFile::Open(LPCTSTR filename, UINT flags, UINT m)
    {
     mode = m;
     flags |= CFile::typeBinary;
     return CFile::Open(filename, flags);
    } // CBiFile::Open

/****************************************************************************
*                             CBiFile::WriteString
* Inputs:
*       const CString & text: Text file
* Result: void
*       
* Effect: 
*       Converts the line according to the desired mode
****************************************************************************/

void CBiFile::WriteString(const CString & text)
    {
     if(text.IsEmpty())
        return;
#ifndef _UNICODE
     Write((LPBYTE)(LPCSTR)text, text.GetLength());
#else
     if(mode == 8)
        { /* UTF-8 conversion */
         CByteArray b;
         int len;
         
         len = WideCharToMultiByte(CP_UTF8,
                                   0,              // must be 0 for UTF conversions
                                   (LPCWSTR)text,  // string to be converted
                                   -1,             // NUL-terminated
                                   NULL,           // no place to put it
                                   0,              // ask to return size
                                   NULL,           // must be NULL for UTF conversions
                                   NULL);          // must be NULL for UTF conversions
         b.SetSize(len);
         len = WideCharToMultiByte(CP_UTF8,
                                   0,              // must be 0 for UTF conversion
                                   (LPCWSTR)text,  // string to be converted
                                   -1,             // NUL-terminated
                                   (LPSTR)b.GetData(),    // place to put it
                                   len,            // length of buffer
                                   NULL,           // must be NULL for UTF conversions
                                   NULL);          // must be NULL for UTF conversions
         Write(b.GetData(), b.GetSize() - 1);
        } /* UTF-8 conversion */
     else
     if(mode == 16 || mode == 0)
        { /* UTF-16 or default */
         Write((LPCWSTR)text, text.GetLength() * sizeof(WCHAR));
        } /* UTF-16 or default */
     else
        { /* failed */
         ASSERT(FALSE);
        } /* failed */
#endif
    } // CBiFile::WriteString
