// ImageView.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "PolyDrawExplorerDoc.h"
#include "ImageView.h"
#include "Pixels.h"
#pragma warning(disable:4312)

// CImageView

IMPLEMENT_DYNCREATE(CImageView, CScrollView)

CImageView::CImageView()
{
}

CImageView::~CImageView()
{
}


BEGIN_MESSAGE_MAP(CImageView, CScrollView)
END_MESSAGE_MAP()


// CImageView drawing

void CImageView::OnInitialUpdate()
{
        CScrollView::OnInitialUpdate();

        CSize sizeTotal;
        // TODO: calculate the total size of this view
        sizeTotal.cx = sizeTotal.cy = 100;
        SetScrollSizes(MM_TEXT, sizeTotal);
}

/****************************************************************************
*                             CImageView::DoDraw
* Inputs:
*       CDC * pDC:
*       BOOL fiducials: TRUE to draw fiduical line at active point, FALSE to not draw it
* Result: void
*       
* Effect: 
*       Draws the image
****************************************************************************/

void CImageView::DoDraw(CDC * pDC, BOOL fiducials)
    {
     CPolyDrawExplorerDoc * pDoc = (CPolyDrawExplorerDoc *)GetDocument();
     BOOL b = pDC->PolyDraw(pDoc->GetPoints(), pDoc->GetTypes(), pDoc->GetCount());
     if(!b)
        { /* failed */
         DWORD err = ::GetLastError();
         pDoc->UpdateAllViews(this, CPolyDrawExplorerDoc::UPDATE_ERROR, (CObject*)err);
        } /* failed */
     else
        { /* success */
         pDoc->UpdateAllViews(this, CPolyDrawExplorerDoc::UPDATE_ERROR, (CObject*)ERROR_SUCCESS);
         if(fiducials)
            { /* draw active point */
             int save = pDC->SaveDC();
             CPen pen(PS_SOLID, 0, RGB(255,0,0));
             pDC->SelectObject(&pen);
#define FIDUCIAL 10
             CPoint pt = pDoc->GetActive();
             pDC->MoveTo(pt.x - FIDUCIAL, pt.y);
             pDC->LineTo(pt.x + FIDUCIAL, pt.y);
             pDC->MoveTo(pt.x, pt.y - FIDUCIAL);
             pDC->LineTo(pt.x, pt.y + FIDUCIAL);
             pDC->RestoreDC(save);
            } /* draw active point */
        } /* success */
    } // CImageView::DoDraw

/****************************************************************************
*                             CImageView::OnDraw
* Inputs:
*       CDC * pDC: DC
* Result: void
*       
* Effect: 
*       Draws the image
****************************************************************************/

void CImageView::OnDraw(CDC* pDC)
   {
    DoDraw(pDC, TRUE);
   }


// CImageView diagnostics

#ifdef _DEBUG
void CImageView::AssertValid() const
{
        CScrollView::AssertValid();
}

void CImageView::Dump(CDumpContext& dc) const
{
        CScrollView::Dump(dc);
}
#endif //_DEBUG



/****************************************************************************
*                       CImageView::WriteMetaFile
* Result: void
*       
* Effect: 
*       Writes a metafile
****************************************************************************/

void CImageView::WriteMetaFile()
   {
    CString filter;                                               
    filter.LoadString(IDS_EMF_FILTER);
    CFileDialog dlg(FALSE, _T(".emf"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, filter);
    if(dlg.DoModal() != IDOK)
       return;

    CString PathName = dlg.GetPathName();

    CString description;
    description = AfxGetApp()->m_pszAppName;

    //--------------------------------------------------------------
    // Desired string, where  is a '\0'
    //
    //   APPNAMEFILENAME
    //   |<--->| |<---->|
    //      |       |
    //      |       Filename.GetLength()
    //      description.GetLength()
    //--------------------------------------------------------------
    int len = description.GetLength();
    LPTSTR p = description.GetBuffer(len + PathName.GetLength() + 3);
    LPTSTR p1 = &p[len + 2];

    //--------------------------------------------------------------
    //   APPNAME............
    //   ^       ^
    //   |       |
    //   p       p1
    //--------------------------------------------------------------
    StringCchCopy(p1, PathName.GetLength(), PathName); // safe because buffer length has been computed properly!

    //--------------------------------------------------------------
    //   APPNAMEFILENAME.
    //   ^       ^       ^
    //   |       |       |
    //   p       p1      &p1[Filename.GetLength()]
    //
    //--------------------------------------------------------------
    p1[PathName.GetLength() + 1] = _T('\0');

    //--------------------------------------------------------------
    //   APPNAMEFILENAME
    //   ^       ^        ^
    //   |       |        |
    //   p       p1       &p1[Filename.GetLength() + 1]
    //
    //--------------------------------------------------------------


    CRect r(0,0,1,1); // TODO: fix this

    // Now convert the pixel units to 0.01mm units

    PixelsTo01mm(r);

    HDC refDC = ::GetDC(NULL);
    HDC metadc = ::CreateEnhMetaFile(refDC, PathName, &r, p);
    if(metadc == NULL)
       { /* failed */
        DWORD err = ::GetLastError();
        GetDocument()->UpdateAllViews(this, CPolyDrawExplorerDoc::UPDATE_ERROR, (CObject*)err);
        ::ReleaseDC(NULL, refDC);
        description.ReleaseBuffer();
        return;
       } /* failed */
    else
       { /* play it, sam */
        DoDraw(CDC::FromHandle(metadc), FALSE);
        description.ReleaseBuffer();
        HENHMETAFILE mf = ::CloseEnhMetaFile(metadc);
        ::DeleteEnhMetaFile(mf);
        ::ReleaseDC(NULL, metadc);
       } /* play it, sam */
   } // CImageView::WriteMetaFile

/****************************************************************************
*                            CImageView::OnUpdate
* Inputs:
*       CView * pSender: ignored
*       LPARAM lHint:
*       CObject * pHint:
* Result: void
*       
* Effect: 
*       Forces various kinds of updates
*
*       lHint           pHint           effect
*       ----------------------------------------------------------------
*       0               NULL            superclass default
*       UPDATE_METAFILE n/a             write metafile
****************************************************************************/

void CImageView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
   {
    switch(lHint)
       { /* lHint */
        case 0:
           if(pHint == NULL)
              CScrollView::OnUpdate(pSender, lHint, pHint);
           return;
        case CPolyDrawExplorerDoc::UPDATE_METAFILE:
           WriteMetaFile();
           return;
       } /* lHint */
   }
