/*****************************************************************************
*           Change Log
*  Date     | Change
*-----------+-----------------------------------------------------------------
* 24-Dec-07 | REQ #054: Created change log
* 24-Dec-07 | REQ #054: Support resizing of rightmost column
*****************************************************************************/
// ListCtrlEx.cpp : implementation file
//

#include "stdafx.h"
#include "ListCtrlEx.h"
#include "BiFile.h"
#include "ToString.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CListCtrlEx

CListCtrlEx::CListCtrlEx()
   {
    Padding = 0;
   }

CListCtrlEx::~CListCtrlEx()
{
}


BEGIN_MESSAGE_MAP(CListCtrlEx, CListCtrl)
        //{{AFX_MSG_MAP(CListCtrlEx)
        ON_WM_SIZE()                                                    // REQ #054
        //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CListCtrlEx message handlers

/****************************************************************************
*                          CListCtrlEx::InsertColumnEx
* Inputs:
*       int n: Column #
*       LPCTSTR caption: Desired caption
*       UINT flags: Flags value
*       int width: (=0) Desired width; 0 means to compute it from the
*               caption. Width of -1 means "fill out the balance"
* Result: int
*       Result of insertcolumn
* Effect: 
*       Inserts the column
****************************************************************************/

int CListCtrlEx::InsertColumnEx(int n, LPCTSTR caption, UINT flags, int width /* = 0 */)
    {
     CClientDC dc(this);
     
     int total = 0;
     for(int i = 0; i < n; i++)
        { /* count all widths */
         total += GetColumnWidth(i);
        } /* count all widths */

     int w = width;
     w += Padding;
     
     if(width == 0)
        w = dc.GetTextExtent(_T(" ") + CString(caption) + _T(" ")).cx + Padding;
     else
     if(width < 0)
        { /* compute balance */
         CRect r;
         GetClientRect(&r);
         w = r.Width() - total + Padding;
         if(w < width)
            w = width; // Do not shorten column so it fits
        } /* compute balance */
     int result = CListCtrl::InsertColumn(n, caption, flags, w);
     return result;
    } // CListCtrlEx::InsertColumnEx

/****************************************************************************
*                          CListCtrlEx::InsertColumnEx
* Inputs:
*       int n: Column #
*       UINT caption: Desired caption as an IDS_ value
*       UINT flags: Flags value
*       int width: (=0) Desired width; 0 means to compute it from the
*               caption. Width of -1 means "fill out the balance"
* Result: int
*       Result of insertcolumn
* Effect: 
*       Inserts the column
****************************************************************************/

int CListCtrlEx::InsertColumnEx(int n, UINT caption, UINT flags, int width /* = 0 */)
    {
     CString s;
     s.LoadString(caption);
     return InsertColumnEx(n, s, flags, width);
    } // CListCtrlEx::InsertColumnEx

/****************************************************************************
*                          CListCtrlEx::InsertColumnEx
* Inputs:
*       int n: Column #
*       LPCTSTR caption: Desired caption
*       UINT flags: Flags value
*       const CString & sample: Sample string
* Result: int
*       return value from InsertColumn
* Effect: 
*       Computes the width of the sample
****************************************************************************/

int CListCtrlEx::InsertColumnEx(int n, LPCTSTR caption, UINT flags, const CString & sample)
    {
     CClientDC dc(this);
     int wc = dc.GetTextExtent(caption).cx;
     int ws = dc.GetTextExtent(sample).cx;
     int w = wc > ws ? wc : ws;
     return CListCtrl::InsertColumn(n, caption, flags, w);
    } // CListCtrlEx::InsertColumnEx

/****************************************************************************
*                          CListCtrlEx::InsertColumnEx
* Inputs:
*       int n: Column #
*       UINT caption: Desired caption as IDS_ value
*       UINT flags: Flags value
*       const CString & sample: Sample string
* Result: int
*       return value from InsertColumn
* Effect: 
*       Computes the width of the sample
****************************************************************************/

int CListCtrlEx::InsertColumnEx(int n, UINT caption, UINT flags, const CString & sample)
    {
     CString s;
     s.LoadString(caption);
     return InsertColumnEx(n, s, flags, sample);
    } // CListCtrlEx::InsertColumnEx

/****************************************************************************
*                              CListCtrlEx::pad
* Inputs:
*       CString & string: String to pad
*       int width: Desired width
*       UINT fmt: Format code
* Result: void
*       
* Effect: 
*       Formats the text
****************************************************************************/

/* static */ void CListCtrlEx::pad(CString & string, int width, UINT fmt)
   {
    if(string.GetLength() >= width)
       return; // no padding possible (should be error)

    int w = width - string.GetLength();
    switch(fmt & LVCFMT_JUSTIFYMASK)
       { /* fmt */
        default:
        case LVCFMT_LEFT:
           string += CString(_T(' '), w);
           break;
        case LVCFMT_RIGHT:
           string = CString(_T(' '), w - 1) + string + _T(' ');
           break;
        case LVCFMT_CENTER:
           { /* split */
            int left = w / 2;
            int right = w - left; // handles odd/even problem
            string = CString(_T(' '), left) + string + CString(_T(' '), right);
           } /* split */
           break;
       } /* fmt */
   } // CListCtrlEx::pad

/****************************************************************************
*                              CListCtrlEx::Save
* Inputs:
*       const CString & filename: File name to save it under
*       UINT mode: Save mode: 0 (default), 8 (UTF-8), 16 (UTF-16)
* Result: BOOL
*       TRUE if successful
*       FALSE if error (use ::GetLastError())
* Effect: 
*       Writes the ListCtrl contents out separated by tabs
****************************************************************************/
class LVCOL {
    public:
       UINT fmt; // indicate left, right, center
       CString title;
       int width; // width in characters
};

BOOL CListCtrlEx::Save(const CString & filename, UINT mode)
    {
     CBiFile f;
     if(!f.Open(filename, CFile::modeWrite | CFile::modeCreate, mode))
        { /* failed */
         return FALSE;
        } /* failed */

     f.WriteBOM();

     CArray<LVCOL, LVCOL> coldata;
     for(int col = 0; /* explicit break */; col++)
        { /* scan columns */
         LVCOLUMN info;
         TCHAR buffer[MAX_PATH];
         info.mask = LVCF_FMT | LVCF_TEXT;
         info.pszText = buffer;
         info.cchTextMax = MAX_PATH;
         if(!GetColumn(col, &info))
            break;
         LVCOL data;
         data.fmt = info.fmt;
         data.title = buffer;
         data.width = data.title.GetLength(); // minimum column width
         coldata.Add(data);
        } /* scan columns */

     // Now scan the columns looking for the widest data item
     for(int row = 0; row < GetItemCount(); row++)
        { /* scan each row */
         for(int col = 0; col < coldata.GetSize(); col++)
            { /* scan each column */
             CString text = GetItemText(row, col);
             if(text.GetLength() > coldata[col].width)
                coldata[col].width = text.GetLength();
            } /* scan each column */
        } /* scan each row */

     // At this point, we know the maximum width needed for each column

     for(col = 0; col < coldata.GetSize(); col++)
        { /* put out headings */
         CString text;
         text = coldata[col].title;
         pad(text, coldata[col].width + 1, coldata[col].fmt);
         f.WriteString(text);
        } /* put out headings */
     static const CString newline = _T("\r\n");
     f.WriteString(newline);
     
     for(row = 0; row < GetItemCount(); row++)
        { /* write each row */
         for(int col = 0; col < coldata.GetSize() ; col++)
            { /* scan each column */
             CString text = GetItemText(row, col);
             pad(text, coldata[col].width + 1, coldata[col].fmt);
             f.WriteString(text);
            } /* scan each column */
         f.WriteString(newline);
        } /* write each row */
     f.Close();
     return TRUE;
    } // CListCtrlEx::Save

/****************************************************************************
*                       CListCtrlEx::SetPeakColumnWidth
* Inputs:
*       int col: Desired column
*       int n: Desired column width
* Result: void
*       
* Effect: 
*       If the value n is greater than the current width, extends the width
****************************************************************************/

void CListCtrlEx::SetPeakColumnWidth(int col, int n)
    {
     int w = GetColumnWidth(col);
     if(w > n)
        return; // already big enough
     CListCtrl::SetColumnWidth(col, n); // update new width
    } // CListCtrlEx::SetPeakColumnWidth

/****************************************************************************
*                       CListCtrlEx::ItemFromPoint
* Inputs:
*       CPoint pt; point where mouse is
* Result: int
*       Index of item in which it is found
****************************************************************************/

int CListCtrlEx::ItemFromPoint(CPoint pt)
    {
     CRect r;
     GetClientRect(&r);
     for(int i = GetTopIndex(); i < GetItemCount() ; i++)
        { /* scan elements */
         CRect g;
         GetItemRect(i, g, LVIR_BOUNDS);
         if(g.top > r.bottom)
            { /* past end */
             return -1; // not found
            } /* past end */
         if(g.PtInRect(pt))
            return i;
        } /* scan elements */
     return -1;
    } // CListCtrlEx::ItemFromPoint

/****************************************************************************
*                       CListCtrlEx::SetColumnAlignment
* Inputs:
*       int col: Column to set
*       int fmt: Desired format
* Result: void
*       
* Effect: 
*       Sets the column alignment
****************************************************************************/

void CListCtrlEx::SetColumnAlignment(int col, int fmt)
    {
     // Key here is that using SetColumn destroys the flag
     // values set for this column in the header control,
     // which could cause the HDF_OWNERDRAW flag to be removed
     CHeaderCtrl * hdr = GetHeaderCtrl();
     HDITEM item = {0};

     if(hdr != NULL)
        { /* has header */
         item.mask = HDI_FORMAT;
         hdr->GetItem(col, &item);
        } /* has header */

     LVCOLUMN coldata = {0};
     coldata.mask = LVCF_FMT;
     coldata.fmt = fmt;
     coldata.iSubItem = col;
     SetColumn(col, &coldata);

     // Now replace the owner-draw flag
     if(hdr != NULL)
        { /* fix up header */
         int od = (item.fmt & HDF_OWNERDRAW);

         hdr->GetItem(col, &item);
         item.fmt |= od;
         hdr->SetItem(col, &item);
        } /* fix up header */
    } // CListCtrlEx::SetColumnAlignment

/****************************************************************************
*                        CListCtrlEx::GetSubItemRectEx
* Inputs:
*       int item: Item #
*       int subitem: Subitem #
*       int area: Aread (LVIR_ code)
*       CRect & r: Rectangle
* Result: BOOL
*       TRUE if successful
*       FALSE if error
* Effect: 
*       Sets the rectangle to the *actual* subitem rectangle.  Handles the
*       undocumented erronroud behavior where column 0 gives the wrong
*       anser
****************************************************************************/

BOOL CListCtrlEx::GetSubItemRectEx(int item, int subitem, int area, CRect & r)
    {
     if(subitem > 0)
        return GetSubItemRect(item, subitem, area, r);

     if(!GetSubItemRect(item, subitem, area, r))
        return FALSE; // failed
     CRect r2;
     if(!GetSubItemRect(item, subitem + 1, area, r2))
        return TRUE;  // only column full column is width of column 0
     r.right = r2.left;
     return TRUE;
    } // CListCtrlEx::GetSubItemRectEx

/****************************************************************************
*                          CListCtrlEx::SetItemEx
* Inputs:
*       int n: Item to insert
*       int sub: subitem
*       LPCTSTR text:
* Result: BOOL
*       
* Effect: 
*       Sets the item specified.  
****************************************************************************/

BOOL CListCtrlEx::SetItemEx(int n, int sub, LPCTSTR text)
    {
     LVITEM subitem = {0};
     subitem.mask = LVIF_TEXT;
     subitem.iItem = n;
     subitem.iSubItem = sub;
     subitem.pszText = (LPTSTR)text;
     return SetItem(&subitem);
    } // CListCtrlEx::SetItemEx

/****************************************************************************
*                      CListCtrlEx::GetColumnHeadingText
* Inputs:
*       int n: Column number
* Result: CString
*       Text of column heading caption
****************************************************************************/

CString CListCtrlEx::GetColumnHeadingText(int n)
    {
     static const int MAX_TEXT = 1024;
     CString s;

     LVCOLUMN col;
     col.mask = LVCF_TEXT;
     col.pszText = s.GetBuffer(MAX_TEXT);
     col.cchTextMax = MAX_TEXT;

     CListCtrl::GetColumn(n, &col);

     s.ReleaseBuffer();
     return s;
    } // CListCtrlEx::GetColumnHeadingText

/****************************************************************************
*                        CListCtrlEx::SetFullRowSelect
* Inputs:
*       BOOL mode: (=TRUE) setting of full-row-select
* Result: void
*       
* Effect: 
*       Sets full-row-select mode
****************************************************************************/

void CListCtrlEx::SetFullRowSelect(BOOL mode /* = TRUE */)
    {
     LPARAM lParam = mode ? LVS_EX_FULLROWSELECT : 0;
     ::SendMessage(m_hWnd, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_FULLROWSELECT, lParam);     
    } // CListCtrlEx::SetFullRowSelect

/****************************************************************************
*                        CListCtrlEx::PreCreateWindow
* Inputs:
*       CREATESTRUCT & cs: Create structure
* Result: BOOL
*       Result of superclass
* Effect: 
*       Sets the LVS_EX_FULLROWSELECT style
****************************************************************************/

BOOL CListCtrlEx::PreCreateWindow(CREATESTRUCT& cs)
    {
     cs.dwExStyle |= LVS_EX_FULLROWSELECT;

     return CListCtrl::PreCreateWindow(cs);
    }

/****************************************************************************
*                             CListCtrlEx::OnSize
* Inputs:
*       UINT nType
*       int cx:
*       int cy:
* Result: void
*       
* Effect: 
*       Stretches the last column to fit the new size, or shrinks it down
****************************************************************************/

void CListCtrlEx::OnSize(UINT nType, int cx, int cy)                // REQ #054
    {                                                               // REQ #054
     CListCtrl::OnSize(nType, cx, cy);                              // REQ #054
                                                                    // REQ #054
     int WidthSoFar = 0;                                            // REQ #054
     int LastColumnWidth = 0;                                       // REQ #054
     for(int i = 0; ; i++)                                          // REQ #054
        { /* scan columns */                                        // REQ #054
         LVCOLUMN column = {0};                                     // REQ #054
         column.mask = LVCF_WIDTH;                                  // REQ #054
         if(!GetColumn(i, &column))                                 // REQ #054
            { /* failed */                                          // REQ #054
             // We failed because there are no more columns, so     // REQ #054
             // WidthSoFar is all the columns up to the last        // REQ #054
             // and LastColumnWidth is the last column width        // REQ #054
             int NewColumnWidth = cx - WidthSoFar;                  // REQ #054
             if(NewColumnWidth < 0)                                 // REQ #054
                break; // don't change if we would go negative      // REQ #054
                                                                    // REQ #054
             column.cx = NewColumnWidth;                            // REQ #054
             SetColumn(i - 1, &column);                             // REQ #054
             break;                                                 // REQ #054
            } /* failed */                                          // REQ #054
         else                                                       // REQ #054
            { /* success */                                         // REQ #054
             WidthSoFar += LastColumnWidth;                         // REQ #054
             LastColumnWidth = column.cx;                           // REQ #054
            } /* success */                                         // REQ #054
        } /* scan columns */                                        // REQ #054
    } // CListCtrlEx::OnSize                                        // REQ #054
