// PolyDrawCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "PolyDrawItem.h"
#include "PolyDrawCtrl.h"
#include "ToString.h"

#ifdef _DEBUG
#define GDI_FLUSH() GdiFlush()
#else
#define GDI_FLUSH()
#endif

// CPolyDrawCtrl

IMPLEMENT_DYNAMIC(CPolyDrawCtrl, CListCtrlEx)
CPolyDrawCtrl::CPolyDrawCtrl()
{
}

CPolyDrawCtrl::~CPolyDrawCtrl()
{
}


BEGIN_MESSAGE_MAP(CPolyDrawCtrl, CListCtrlEx)
    ON_NOTIFY_REFLECT(LVN_DELETEITEM, OnLvnDeleteitem)
END_MESSAGE_MAP()



// CPolyDrawCtrl message handlers


void CPolyDrawCtrl::OnLvnDeleteitem(NMHDR *pNMHDR, LRESULT *pResult)
    {
     LPNMLISTVIEW pNMLV = reinterpret_cast<LPNMLISTVIEW>(pNMHDR);
     PolyDrawItem * pi = (PolyDrawItem *)pNMLV->lParam;
     delete pi;
     *pResult = 0;
    }

/****************************************************************************
*                           CPolyDrawCtrl::DrawItem
* Inputs:
*       LPDRAWITEMSTRUCT dis: 
* Result: void
*       
* Effect: 
*       Draws the item
****************************************************************************/

void CPolyDrawCtrl::DrawItem(LPDRAWITEMSTRUCT dis)
    {
     CRect r;
     CDC * dc = CDC::FromHandle(dis->hDC);
     int save = dc->SaveDC();

     LVCOLUMN colinfo = {0};

     colinfo.mask = LVCF_FMT | LVCF_WIDTH;
     COLORREF fill = RGB(255,255,255);
     COLORREF text;
     UINT dtformat = 0;

     int gap = 2 * ::GetSystemMetrics(SM_CXBORDER); 
     int offset = -GetScrollPos(SB_HORZ);

     //----------------------------------------------------------------
     // Compute the colors for fill and text
     //----------------------------------------------------------------
     if(dis->itemState & ODS_SELECTED)
        { /* selected */
         if(::GetFocus() == m_hWnd)
            { /* has focus */
             fill = ::GetSysColor(COLOR_HIGHLIGHT);
             text = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
            } /* has focus */
         else
            { /* no focus */
             fill = ::GetSysColor(COLOR_GRAYTEXT);
             text = ::GetSysColor(COLOR_WINDOWTEXT);
            } /* no focus */
        } /* selected */
     else
        { /* not selected */
#define BAND1_COLOR RGB(192,255,192)
#define BAND2_COLOR RGB(155,255,155)
#define DISABLED_BAND1_COLOR RGB(192,192,192)
#define DISABLED_BAND2_COLOR RGB(155,155,155)
         COLORREF band1;
         COLORREF band2;
         if(IsWindowEnabled())
            { /* enabled */
             text = ::GetSysColor(COLOR_WINDOWTEXT);
             band1 = BAND1_COLOR;
             band2 = BAND2_COLOR;
            } /* enabled */
         else
            { /* disabled */
             text = RGB(128,128,128);
             band1 = DISABLED_BAND1_COLOR;
             band2 = DISABLED_BAND2_COLOR;
            } /* disabled */

         switch(dis->itemID % 6)
            { /* color selection */
             case 0:
             case 1:
             case 2:
                fill = band1;
                break;
             case 3:
             case 4:
             case 5:
                fill = band2;
                break;
            } /* color selection */

        } /* not selected */

     for(int col = 0; GetColumn(col, &colinfo); col++)
        { /* scan columns */
         //----------------------------------------------------------------
         // Compute the rectangle
         //----------------------------------------------------------------
         GetSubItemRectEx(dis->itemID, col, LVIR_BOUNDS, r);
         int pos = r.left;

         dc->SetTextColor(text);

         //----------------------------------------------------------------
         // Fill the background. Note the adjustment down and the expansion
         // This looks better
         //----------------------------------------------------------------
         CRect filler = r;
         //filler.top += 1;
         filler.bottom += 2;

         dc->FillSolidRect(&filler, fill);
         GDI_FLUSH();

         //----------------------------------------------------------------
         // Compute the correct width of the column
         // Note that GetItemRect gives then ENTIRE rectangle, and
         // getting the subitem rect for item 0 does the same thing
         // So if we are in the first column, we ask for the second
         // column
         //----------------------------------------------------------------
         int width = r.Width();

         //----------------------------------------------------------------
         // Now compute the alignment. Note that if there is 
         // horizontal scrolling, we have to adjust for the scrolling
         // position
         //----------------------------------------------------------------

         int x; // = dis->rcItem.left + pos + offset;

         //----------------------------------------------------------------
         // Set up to honor the formatting alignment
         //----------------------------------------------------------------
         switch(colinfo.fmt & LVCFMT_JUSTIFYMASK)
            { /* fmt */
             case LVCFMT_CENTER:
                x = dis->rcItem.left + pos + width / 2 - offset;
                dc->SetTextAlign(TA_CENTER);
                dtformat = DT_CENTER;
                break;
             case LVCFMT_RIGHT:
                x = dis->rcItem.left + pos + width - offset - gap;
                dc->SetTextAlign(TA_RIGHT);
                dtformat = DT_RIGHT;
                break;
             case LVCFMT_LEFT:
             default:
                x = dis->rcItem.left + pos - offset;
                dc->SetTextAlign(TA_LEFT);
                dtformat = DT_LEFT;
                break;
            } /* fmt */

         //----------------------------------------------------------------
         // Set a clipping region so we don't paint outside the lines...
         // Note that we have to add +1 to deal with the fact that all
         // drawing is up-to-but-not-including the endpoint, and we
         // want to include the endpoint
         //----------------------------------------------------------------

         CRect clip(r.left, r.top, r.right - gap, r.bottom + 1);

         CRgn cliprgn;
         cliprgn.CreateRectRgn(clip.left, clip.top, clip.right, clip.bottom);
         { /* draw clipped */
          int save2 = dc->SaveDC();
          dc->SelectClipRgn(&cliprgn);

          //----------------------------------------------------------------
          // There appears to be a serious bug in the DrawText API, in that
          // it gives erroneous positioning when anything other than DT_LEFT
          // is specified!  For example, a sequence of right-justified
          // numbers oves out offset by the number of digits, such as
          //             8
          //             9
          //           10
          //           12
          //           50
          //         100
          //         500
          //       1000
          //       1000
          //       5000
          //     10000
          // Of course, this makes no sense; I've single-stepped through this
          // code and I'm passing in the correct parameters, and it still gets
          // it wrong.  But I want the automatic ellipses, so what I've chosen
          // to do here is use DrawText only when DT_LEFT is selected
          // For reasons not entirely understood, I get the wrong computation
          // returned for GetItemRect(0), so I special case col == 0 as well

          PolyDrawItem * pi = (PolyDrawItem*)dis->itemData;
          CString s;
          CFont font;

          switch(col)
             { /* col */
              case COL_NUM:
                 s = ToString(_T("%d"), dis->itemID);
                 break;
              case COL_X:
                 s = ToString(_T("%d"), pi->pt.x);
                 break;
              case COL_Y:
                 s = ToString(_T("%d"), pi->pt.y);
                 break;
              case COL_TYPE:
                 if( (pi->type & ~PT_CLOSEFIGURE) == PT_MOVETO)
                    s = _T("PT_MOVETO");
                 else
                 if( (pi->type & ~PT_CLOSEFIGURE) == PT_LINETO)
                    s = _T("PT_LINETO");
                 else
                 if( (pi->type & ~PT_CLOSEFIGURE) == PT_BEZIERTO)
                    s = _T("PT_BEZIERTO");
                 break;
              case COL_CLOSEFIGURE:
                 { /* closefigure */
                  if( pi->type & PT_CLOSEFIGURE)
                     s = _T("\xFE");
                  else
                     s = _T("\x6F");
                  LOGFONT lf;
                  CFont * f = GetFont();
                  f->GetLogFont(&lf);
                  lf.lfPitchAndFamily = FF_DECORATIVE;
                  lf.lfCharSet = SYMBOL_CHARSET;
                  StringCchCopy(lf.lfFaceName, sizeof(lf.lfFaceName)/sizeof(TCHAR), _T("Wingdings"));
                  font.CreateFontIndirect(&lf);
#ifdef _DEBUG
                  font.GetLogFont(&lf);
#endif
                  dc->SelectObject(&font);
                 } /* closefigure */
                 break;
             } /* col */

          CRect dtrect = clip;
          dtrect.left += offset;
          dtrect.right += offset;
          dtrect.top--;

          if(col > 0 && dtformat == DT_LEFT)
             dc->DrawText(s, &dtrect, dtformat | DT_WORD_ELLIPSIS);
          else
             dc->TextOut(x, dis->rcItem.top - 1, s);
          dc->RestoreDC(save2);
         } /* draw clipped */
         //pos += width;
        } /* scan columns */
     dc->RestoreDC(save);
     
    } // CPolyDrawCtrl::DrawItem

/****************************************************************************
*                          CPolyDrawCtrl::Invalidate
* Inputs:
*       int item: Item to invalidate
* Result: void
*       
* Effect: 
*       Invalidates the item
****************************************************************************/

void CPolyDrawCtrl::Invalidate(int item)
    {
     CRect r;
     GetItemRect(item, &r, LVIR_BOUNDS);
     InvalidateRect(&r);
    } // CPolyDrawCtrl::Invalidate

/****************************************************************************
*                          CPolyDrawCtrl::InsertItem
* Inputs:
*       PolyDrawItem * pi:
* Result: int
*       
* Effect: 
*       Inserts the item at the end of the list
****************************************************************************/

int CPolyDrawCtrl::InsertItem(PolyDrawItem * pi)
    {
     LVITEM item = {0};
     item.mask = LVIF_PARAM;
     item.lParam = (LPARAM)pi;
     item.iItem = GetItemCount();
     return CListCtrlEx::InsertItem(&item);     
    } // CPolyDrawCtrl::InsertItem
